{-----------------------------------------------------------------------------

 CF_Grph : main graphical interface of the game ChiFu.

 Consists of:

 Types:
 -TScreen : standard screen interface definition
   Variables:
   -CursorPos  : the current cursor position
   -TitlePos   : standard title display position
   -ScorePos   : standard score display position
   -Home       : home cursor position
   -Middle     : middle cursor position
   -Bottom     : bottom cursor position
   -OptionPos  : standard screen positions of all options

   Routines:
   -Init             : initializes the screen interface
   -ClearScreen      : clears te screen
   -ReturnTextMode   : returns the screen to text mode
   -DrawBorder       : draws the border
   -DrawField        : draws the field on screen
   -DrawBlock        : draws the block on screen
   -DeleteBlock      : deletes the block from screen
   -ShowText         : shows the given TextLine at the current location
   -ShowCursor       : shows the cursor
   -HideCursor       : hides the cursor from sight
   -ScrollUp         : scrolls the cursor up one column
   -ScrollDown       : scrolls the cursor down one column
   -ScrollLeft       : scrolls the cursor left one column
   -ScrollRight      : scrolls the cursor right one column
   -ShowArrow        : shows the given arrow at the CursorPos
   -ShowPauseScreen  : shows the pause sreen
   -ShowTitleScreen  : shows the title screen
   -ShowOptionBox    : shows a box around the given option
   -DeleteOptionBox  : deletes the box around the given option
   -ShowSettings     : shows the current settings

 Constants:
 -LeftArrow,
 -RightArrow  : character codes for the arrows

 Variables:
 -Screen : the screen interface

 Routines:

 Defaults:
 -Initializes the screen interface

-----------------------------------------------------------------------------}
unit CF_Grph;

interface

uses
  CF_Play,
  CF_Set,
  CF_Pict,
  Graph;

const
  { All arrow character codes }
  LeftArrow  : char = #127;
  Rightarrow : char = #128;

type
  TScreen = object
              public
                CursorPos,             { The current cursor position }
                TitlePos,              { Standard title display position }
                ScorePos,              { Standard score display position }
                Home,                  { Home cursor position }
                Middle,                { Middle cursor position }
                Bottom      : TCoord;  { Bottom cursor position }
                OptionPos   : array [TOption] of TCoord;
                                       { Standard positions of all options }
                OptionWidth : array [TOption] of word;
                                       { Standard option box widths }

                { Initializes the screen interface }
                constructor Init;

                { Clears te screen }
                procedure ClearScreen;

                { Returns the screen to text mode }
                procedure ReturnTextMode;

                { Draws the border }
                procedure DrawBorder;

                { Draws the field on screen }
                procedure DrawField;

                { Draws the block on screen }
                procedure DrawBlock;

                { Deletes the block from screen }
                procedure DeleteBlock;

                { Shows the given TextLine at the current location }
                procedure ShowText (TextLine : string);

                { Shows the cursor }
                procedure ShowCursor;

                { Hides the cursor from sight }
                procedure HideCursor;

                { Scrolls the cursor up one column }
                procedure ScrollUp;

                { Scrolls the cursor down one column }
                procedure ScrollDown;

                { Scrolls the cursor left one column }
                procedure ScrollLeft;

                { Scrolls the cursor right one column }
                procedure ScrollRight;

                { Shows the pause sreen }
                procedure ShowPauseScreen;

                { Shows the title screen }
                procedure ShowTitleScreen;

                { Shows a box around the given option }
                procedure ShowOptionBox (Option : TOption);

                { Deletes the box around the given option }
                procedure DeleteOptionBox (Option : TOption);

              private
                PrevCursorPos,          { Prev. cursor position }
                LeftUpField   : TCoord; { Left upper corner playfield }

                MaxX,
                MaxY          : word;   { Maximum screen size }

                CursorVisible : boolean;

                { Initializes the screen into graphics-mode }
                procedure InitializeScreen;

                { Transfers the given FieldCoord. to the right ScreenCoord. }
                procedure MakeScreenCoord (var FieldCoord, ScreenCoord : TCoord);

                { Draws the given stone }
                procedure DrawStone (Stone : TStone);

                { Shows the cursor at the current location, and deletes the
                  cursor at the previous location. Also makes the current
                  location the previous one. }
                procedure PlotCursor;

            end; { object }

var
  Screen : TScreen;

implementation

uses
  CF_KIO;

{ Initializes the screen interface }
constructor TScreen.Init;

begin
  InitializeScreen;
  LeftUpField.X := 20 - StoneSize;
  LeftUpField.Y := -StoneSize;

  Home.X := 0;
  Home.Y := 0;
  Middle.X := 0;
  Middle.Y := MaxY div 2;
  Bottom.X := 0;
  Bottom.Y := MaxY - LetterHeight;
  ScorePos.X := 300;
  ScorePos.Y := 50;
  TitlePos.X := 300;
  TitlePos.Y := 10;

  OptionPos [PlayGame].X := 372;
  OptionPos [PlayGame].Y := 105;
  OptionWidth [PlayGame] := 158;
  OptionPos [ChangeFullRows].X := 300;
  OptionPos [ChangeFullRows].Y := 140;
  OptionWidth [ChangeFullRows] := 328;
  OptionPos [SoundOnOff].X := 354;
  OptionPos [SoundOnOff].Y := 175;
  OptionWidth [SoundOnOff] := 192;
  OptionPos [ChangePlayMode].X := 312;
  OptionPos [ChangePlayMode].Y := 210;
  OptionWidth [ChangePlayMode] := 276;
  OptionPos [ShowHi].X := 348;
  OptionPos [ShowHi].Y := 245;
  OptionWidth [ShowHi] := 204;
  OptionPos [Instructions].X := 348;
  OptionPos [Instructions].Y := 280;
  OptionWidth [Instructions] := 204;
  OptionPos [ExitGame].X := 396;
  OptionPos [ExitGame].Y := 315;
  OptionWidth [ExitGame] := 108;

  CursorVisible := false;
  CursorPos := Home;
  PrevCursorPos := Home;
end; { Init }

{ Transfers the given FieldCoord. to the right ScreenCoord. }
procedure TScreen.MakeScreenCoord (var FieldCoord, ScreenCoord : TCoord);

begin
  ScreenCoord.X := LeftUpField.X + FieldCoord.X * StoneSize + (11 - Settings [FieldWidth]) div 2 * StoneSize;
  ScreenCoord.Y := LeftUpField.Y + FieldCoord.Y * StoneSize;
end; { MakeScreenCoord }

{ Initializes the screen into graphics-mode }
procedure TScreen.InitializeScreen;

var
  Driver,
  Mode : integer;

begin
  { Put screen in graphics-mode }
  Driver := Detect;
  InitGraph (Driver, Mode, '.');
  MaxX := GetMaxX;
  MaxY := GetMaxY;
end; { Initialize }

{ Returns the screen to text mode }
procedure TScreen.ReturnTextMode;

begin
  RestoreCrtMode;
end; { ReturnTextMode }

{ Clears te screen }
procedure TScreen.ClearScreen;

begin
  ClearViewPort;
end; { ClearScreen }


{ Draws the border }
procedure TScreen.DrawBorder;

begin
  SetColor (14);
  Line (LeftUpField.X - 2 + StoneSize +
        (11 - Settings [FieldWidth]) div 2 * StoneSize,
        LeftUpField.Y + StoneSize,
        LeftUpField.X - 2 + StoneSize +
        (11 - Settings [FieldWidth]) div 2 * StoneSize,
        LeftUpField.Y + (Settings [FieldLength] + 1) * StoneSize + 2);
  Line (LeftUpField.X - 2 + StoneSize +
        (11 - Settings [FieldWidth]) div 2 * StoneSize,
        LeftUpFIeld.Y + (Settings [FieldLength] + 1) * StoneSize + 2,
        LeftUpField.X + (Settings [FieldWidth] + 1) * StoneSize + 2 +
        (11 - Settings [FieldWidth]) div 2 * StoneSize,
        LeftUpField.Y + (Settings [FieldLength] + 1) * StoneSize + 2);
  Line (LeftUpField.X + (Settings [FieldWidth] + 1) * StoneSize + 2 +
        (11 - Settings [FieldWidth]) div 2 * StoneSize,
        LeftUpField.Y + (Settings [FieldLength] + 1) * StoneSize + 2,
        LeftUpField.X + (Settings [FieldWidth] + 1) * StoneSize + 2 +
        (11 - Settings [FieldWidth]) div 2 * StoneSize,
        LeftUpFIeld.Y + StoneSize);
end; { DrawBorder }


{ Draws the field on screen }
procedure TScreen.DrawField;

var
  FieldCoord : TCoord;
  FieldStone : TStone;

begin
  { For every field coordinate }
  for FieldCoord.X := 1 to Settings [FieldWidth] do begin
    for FieldCoord.Y := 1 to Settings [FieldLength] do begin
      { Get the stone in the field }
      Field.GetStone (FieldCoord, FieldStone);

      { And draw that stone }
      DrawStone (FieldStone);
    end; { for }
  end; { for }
end; { DrawField }

{ Draws the given stone }
procedure TScreen.DrawStone (Stone : TStone);

var
  ScreenCoord : TCoord;

begin
  { Get the right screen coordinates }
  MakeScreenCoord (Stone.Coord, ScreenCoord);

  { And draw the stone }
  if Stone.Sortof = NoStone then begin
    { Make board background }
    SetFillStyle (1, 0);
    Bar (ScreenCoord.X + 1, ScreenCoord.Y + 1,
         ScreenCoord.X + StoneSize - 1, ScreenCoord.Y + StoneSize - 1);
    SetColor (8);
    Rectangle (ScreenCoord.X, ScreenCoord.Y,
               ScreenCoord.X + StoneSize, ScreenCoord.Y + StoneSize);
  end { then }
  else begin
    { Draw stone }
    SetColor (0);
    Rectangle (ScreenCoord.X + 1, ScreenCoord.Y + 1,
               ScreenCoord.X + StoneSize - 1, ScreenCoord.Y + StoneSize - 1);
    PutImage (ScreenCoord.X + 1, ScreenCoord.Y + 1, StonePict [Stone.Variety]^, NormalPut);
  end; { else }
end; { DrawStone }

{ Draws the block on screen }
procedure TScreen.DrawBlock;

var
  StoneNr    : byte;
  BlockStone : TStone;

begin
  { For every stone in the block }
  for StoneNr := 1 to Settings [BlockLength] do begin
    { Get the stone in the block }
    Block.GetStone (StoneNr, BlockStone);

    { And draw that stone }
    DrawStone (BlockStone);
  end; { for }
end; { DrawBlock }

{ Deletes the block from screen }
procedure TScreen.DeleteBlock;

var
  StoneNr    : byte;
  BlockStone : TStone;

begin
  { For every stone in the block }
  for StoneNr := 1 to Settings [BlockLength] do begin
    { Get the stone in the block }
    Block.GetStone (StoneNr, BlockStone);

    { Convert it to an empty stone }
    BlockStone.Sortof := NoStone;

    { And draw that stone }
    DrawStone (BlockStone);
  end; { for }
end; { DeleteBlock }

{ Shows the given TextLine at the current location }
procedure TScreen.ShowText (TextLine : string);

var
  Position : TCoord;
  Letter   : byte;

begin
  { Write the actual text }
  Position := CursorPos;
  for Letter := 1 to Length (TextLine) do begin
    PutImage (Position.X, Position.Y,
              CharPict [byte (TextLine [Letter]) - 32]^, NormalPut);
    Position.X := Position.X + LetterWidth;
  end; { for }
end; { ShowText }

{ Shows the cursor }
procedure TScreen.ShowCursor;

begin
  CursorVisible := true;
  PlotCursor;
end; { ShowCursor }

{ Hides the cursor from sight }
procedure TScreen.HideCursor;

begin
  { Set the controller }
  CursorVisible := false;

  { And delete the cursor from sight }
  SetColor (0);
  Line (CursorPos.X + 1, CursorPos.Y + LetterHeight,
        CursorPos.X + LetterWidth - 1, CursorPos.Y + LetterHeight);
end; { HideCursor }

{ Shows the cursor at the current location, and deletes the cursor at the
  previous location. Also makes the current location the previous one. }
procedure TScreen.PlotCursor;

begin
  SetColor (0);
  Line (PrevCursorPos.X + 1, PrevCursorPos.Y + LetterHeight,
        PrevCursorPos.X + LetterWidth - 1, PrevCursorPos.Y + LetterHeight);
  SetColor (14);
  Line (CursorPos.X + 1, CursorPos.Y + LetterHeight,
        CursorPos.X + LetterWidth - 1, CursorPos.Y + LetterHeight);
  PrevCursorPos := CursorPos;
end; { PlotCursor }

{ Scrolls the cursor left one column }
procedure TScreen.ScrollLeft;

begin
  PrevCursorPos := CursorPos;
  CursorPos.X := CursorPos.X - LetterWidth;
  if CursorVisible then begin
    PlotCursor;
  end; { then }
end; { ScrollLeft }

{ Scrolls the cursor right one column }
procedure TScreen.ScrollRight;

begin
  PrevCursorPos := CursorPos;
  CursorPos.X := CursorPos.X + LetterWidth;
  if CursorVisible then begin
    PlotCursor;
  end; { then }
end; { ScrollRight }

{ Scrolls the cursor down one column }
procedure TScreen.ScrollDown;

begin
  PrevCursorPos := CursorPos;
  CursorPos.Y := CursorPos.Y + LetterWidth;
  if CursorVisible then begin
    PlotCursor;
  end; { then }
end; { ScrollRight }

{ Scrolls the cursor up one column }
procedure TScreen.ScrollUp;

begin
  PrevCursorPos := CursorPos;
  CursorPos.Y := CursorPos.Y - LetterWidth;
  if CursorVisible then begin
    PlotCursor;
  end; { then }
end; { ScrollUp }

{ Shows a box around the given option }
procedure TScreen.ShowOptionBox (Option : TOption);

begin
  SetColor (14);
  Rectangle (OptionPos [Option].X - 16,
             OptionPos [Option].Y - 4,
             OptionPos [Option].X + OptionWidth [Option] + 16,
             OptionPos [Option].Y + LetterHeight + 4);
end; { ShowOptionBox }

{ Deletes the box around the given option }
procedure TScreen.DeleteOptionBox (Option : TOption);

begin
  SetColor (0);
  Rectangle (OptionPos [Option].X - 16,
             OptionPos [Option].Y - 4,
             OptionPos [Option].X + OptionWidth [Option] + 16,
             OptionPos [Option].Y + LetterHeight + 4);
end; { DeleteOptionBox }

{ Shows the title screen }
procedure TScreen.ShowTitleScreen;

var
  Variety,
  StoneNr : byte;
  Coord   : TCoord;
  Stone   : TStone;

begin
  { Show the main picture }
  PutImage (175, 145, TitlePict^, NormalPut);

  { And put in some random stones }
  Randomize;
  for StoneNr := 0 to 29 do begin
    { Make a random position }
    repeat
      Coord.X := Random (MaxX - 50) + 10;
      Coord.Y := Random (MaxY - 50) + 10;
    until (Coord.Y < 115) or (Coord.Y > 330) or (Coord.X < 140) or (Coord.X > 480);

    { And a random stone }
    Variety := Random (AmDiffStones) + 1;

    { And show it }
    PutImage (Coord.X, Coord.Y, StonePict [Variety]^, NormalPut);
  end; { for }
end; { ShowTitleScreen }





{ Pause screen definition----------------------------------------------------}

type
  { Real polar coordinate definition }
  TPolarCoord = record
                  R,
                  Fi : real;
                end; { record }

  { Single dot definition }
  TDot = record
           Coord : TPolarCoord;
           Speed : real;
           Color : word;
         end; { record }

{ Shows the pause sreen }
procedure TScreen.ShowPauseScreen;

{ Transforms the given polar coordinates into screen coordinates }
procedure PolarToScreen (PolarCoord : TPolarCoord; var ScreenCoord : TCoord);

begin
  ScreenCoord.X := MaxX div 2 + Trunc (PolarCoord.R * Cos (PolarCoord.Fi));
  ScreenCoord.Y := MaxY div 2 - Trunc (PolarCoord.R * Sin (PolarCoord.Fi));
end; { PolarToScreen }

{ Resets the given dot }
procedure ResetDot (var Dot: TDot);

begin
  with Dot do begin
    Coord.Fi := Random (Trunc (2000 * Pi)) / 1000;
    Coord.R  := 0;
    Speed    := Random (4000) / 1000 + 4;
    Color    := 11 + Random (2) + Random (2) * 2;
  end; { with }
end; { ResetDot }

var
  Count       : byte;
  Dot         : array [0..75] of TDot;
  ScreenCoord : TCoord;

begin
  { Clear the screen }
  ClearScreen;

  { Make all new dots }
  for Count := 0 to 75 do begin
    ResetDot (Dot [Count]);
  end; { for }

  { And start the moving }
  repeat
    { For every dot }
    for Count := 0 to 75 do begin
      with Dot [Count] do begin
        { Remove the dot at it's previous location }
        SetColor (0);
        PolarToScreen (Coord, ScreenCoord);
        Circle (ScreenCoord.X, ScreenCoord.Y, 1);

        { Move the dot }
        Coord.Fi := Coord.Fi + Coord.R / 2400;
        coord.R  := Coord.R + Speed * (Coord.R / 800 + 1);

        { Reset it if it is moving out of screen }
        if Coord.R > MaxX then begin
          ResetDot (Dot [Count]);
        end; { then }

        { And show the dot }
        SetColor (Color);
        PolarToScreen (Coord, ScreenCoord);
        Circle (ScreenCoord.X, ScreenCoord.Y, 1);
      end; { with }
    end; { for }
  until Keyboard.KeyPressed;
  Keyboard.GetKey;

  ClearScreen;
end; { dots }

begin
  Screen.Init;
end. { CF_Grph }