{-----------------------------------------------------------------------------

 KIO : keyboard I/O interface

 Consists of:

 Types:
 -TKIO : keyboard I/O interface definition
   Variables:
   -CaseOn : whether or not to at case-sensitive

   Routines:
   -Init           : initialization of the keyboard I/O interface
   -GetKey         : lets the user enter a key on the keyboard
   -GetSelectedKey : enables the user to enter one of the selected keys and
                     gives back this key
   -WaitKeyPress   : traps program flow until a key has been pressed
   -KeyPressed     : whether or not a key has been pressed
   -EmptyBuffer    : empties the keyboard buffer
   -GetText        : gets text from the user

 Constants:
 -Home,
 -Endk
 -Enter,
 -Esc,
 -BckSpc,
 -Del,
 -Up,
 -Down,
 -Left,
 -Right : standard key codes (returned by the GetKey routines)

 Variables:
 -KeyBoard : main keyboard interface implementation

 Routines:

 Defaults:
 -Initializes the keyboard interface

-----------------------------------------------------------------------------}
unit CF_KIO;

interface

uses
  CF_Grph,
  CF_Play,
  Crt;

const
  Enter  : char = #13;
  Esc    : char = #27;
  BckSpc : char = #08;
  Del    : char = #20;

  Up    : char = #14;
  Down  : char = #15;
  Left  : char = #16;
  Right : char = #17;

  Home : char = #18;
  Endk : char = #19;

type

  { TKIO : main keyboard I/O interface definition and variables }
  TKIO = object
           public
             { Whether or not to at case-sensitive }
             CaseOn : boolean;

             { Initialization of the keyboard I/O interface }
             constructor Init;

             { Lets the user enter a key on the keyboard }
             function GetKey : char;

             { Enables the user to enter one of the selected keys and gives
               back this key }
             function GetSelectedKey (Selection : string) : char;

             { Traps program flow until a key has been pressed }
             procedure WaitKeyPress;

             { Whether or not a key has been pressed }
             function KeyPressed : boolean;

             { Empties the keyboard buffer }
             procedure EmptyBuffer;

             { Gets text from the user }
             function GetText : string;

           private

         end; { object }

var
  Keyboard : TKIO;

implementation

{ LowerString : turns the given string to all lower-case }
function LowerString (InString : string) : string;

var
  LetterNr : byte;

begin
  for LetterNr := 1 to Length (InString) do begin
    if (InString [LetterNr] >= 'A') and (InString [LetterNr] <= 'Z') then begin
      InString [LetterNr] := char (byte (InString [LetterNr]) + 32);
    end; { then }
  end; { for }

  LowerString := InString;
end; { LowerString }

{ LowerChar : turns the given char to lower-case }
function LowerChar (InChar : char) : char;

begin
  if (InChar >= 'A') and (InChar <= 'Z') then begin
    LowerChar := char (byte (InChar) + 32);
  end { then }
  else begin
    LowerChar := InChar;
  end; { else }
end; { LowerChar }

{ Init : initialization of the keyboard I/O interface }
constructor TKIO.Init;

begin
  CaseOn := false;
end; { Init }

{ GetKey : lets the user enter a key on the keyboard }
function TKIO.GetKey : char;

var
  InChar : char;

begin
  repeat until Crt.KeyPressed;
  InChar := ReadKey;
  if InChar = #00 then begin
    InChar := ReadKey;
    case InChar of
      #72 : InChar := Up;
      #75 : InChar := Left;
      #77 : InChar := Right;
      #80 : InChar := Down;

      #71 : InChar := Home;
      #79 : InChar := Endk;
      #83 : InChar := Del;
    end; { case }
  end; { then }

  if CaseOn then begin
    GetKey := InChar;
  end { then }
  else begin
    GetKey := LowerChar (InChar);
  end; { else }
end; { GetKey }

{ GetSelectedKey : enables the user to enter one of the selected keys and
                   gives back this key }
function TKIO.GetSelectedKey (Selection : string) : char;

var
  InChar     : char;
  OptionNr   : byte;
  RightInput : boolean;

begin
  if not CaseOn then begin
    Selection := LowerString (Selection);
  end; { then }

  RightInput := false;
  repeat
    InChar := GetKey;
    for OptionNr := 1 to Length (Selection) do begin
      if InChar = Selection [OptionNr] then begin
        RightInput := true;
      end; { then }
    end; { for }
  until RightInput;

  GetSelectedKey := InChar;
end; { GetSelectedKey }

{ WaitKeyPress : traps program flow until a key has been pressed }
procedure TKIO.WaitKeyPress;

begin
  repeat until Crt.KeyPressed;
end; { WaitKeyPress }

{ KeyPressed : whether or not a key has been pressed }
function TKIO.KeyPressed : boolean;

begin
  KeyPressed := Crt.KeyPressed;
end; { KeyPressed }

{ GetText : gets text from the user }
function TKIO.GetText : string;

var
  TextStart : TCoord;  { Start of textline on screen }
  LinePos   : byte;    { Place of cursor in line }
  InChar    : char;    { Used to read char's from keyboard }
  EndRead   : boolean; { End of reading requested }
  TextLine  : string;  { To use during modifications }

begin
  { Write the given TextLine to screen }
  Screen.ShowCursor;
  TextLine := '';
  TextStart := Screen.CursorPos;
  EndRead := false;
  LinePos := 1;

  repeat
    { Get response from user }
    InChar := GetKey;

    { And act according to that response }
    if InChar = Right then begin
      { Look if cursor can scroll }
      if LinePos < Length (TextLine) + 1 then begin
        { Yes -> do so }
        LinePos := LinePos + 1;
        Screen.ScrollRight;
      end; { then }
    end { then }
    else if InChar = Left then begin
      { See if cursor can scroll }
      if LinePos > 1 then begin
        { Yes -> do so }
        LinePos := LinePos - 1;
        Screen.ScrollLeft;
      end; { then }
    end { else then }
    else if InChar = Del then begin
      { Delete char. at cursorpos. }
      Delete (TextLine, LinePos, 1);

      { And show the new situation }
      Screen.ShowText (Copy (TextLine, LinePos, Length (TextLine)) + ' ');

      { And show the cursor again }
      Screen.ScrollRight;
      Screen.ScrollLeft;
    end { else then }
    else if InChar = Home then begin
      Screen.HideCursor;
      while LinePos <> 1 do begin
        Screen.ScrollLeft;
        Dec (LinePos);
      end; { while }
      Screen.ShowCursor;
    end { else then }
    else if InChar = Endk then begin
      Screen.HideCursor;
      while LinePos < Length (TextLine) + 1 do begin
        Screen.ScrollRight;
        Inc (LinePos);
      end; { while }
      Screen.ShowCursor;
    end { else then }
    else if InChar = Enter then begin
      EndRead := true;
    end { else then }
    else if InChar = BckSpc then begin
      { See if key is valid in situation }
      if LinePos > 1 then begin
        { Delete the char. in front of cursor }
        Delete (TextLine, LinePos - 1, 1);

        { Move the cursor }
        Dec (LinePos);
        Screen.ScrollLeft;

        { Show the new situation }
        Screen.ShowText (Copy (TextLine, LinePos, Length (TextLine)) + '  ');

        { And show the cursor again }
        Screen.ScrollRight;
        Screen.ScrollLeft;
      end; { then }
    end { else then }
    else if (InChar >= #32) and (InChar <= #128) then begin
      if Length (TextLine) < 15 then begin
        { Insert the keystroke in the textline }
        Insert (InChar, TextLine, LinePos);

        { And show the new situation }
        Screen.ShowText (Copy (TextLine, LinePos, Length (TextLine)) + ' ');

        { Move the cursor }
        Inc (LinePos);
        Screen.ScrollRight;
      end;  { then }
    end; { else then }
  until EndRead;

  Screen.HideCursor;
  GetText := TextLine;
end; { GetText }

{ Empties the keyboard buffer }
procedure TKIO.EmptyBuffer;

begin
  while Crt.KeyPressed do begin
    ReadKey;
  end; { while }
end; { EmptyBuffer }

begin
  KeyBoard.Init;
end. { CF_KIO }