{-----------------------------------------------------------------------------

 CF_Snd : sound definition and implementation of the game ChiFu.

 Consists of:

 Types:
 -TSound : sound interface definition

 Constants:

 Variables:
 -Speaker : the internal speaker

 Routines:

 Defaults:
 -Initializes the speaker

-----------------------------------------------------------------------------}
unit CF_Snd;

interface

uses
  CF_Set,
  Crt;

const
  { Maximum amount of tunes }
  MaxTunes = 9;

type
  { A single tune element }
  TTuneElem = record
                Freq  : word;
                Delay : byte;
              end; { record }

  { And convienient pointer look-a-like }
  PTuneElem = ^TTuneElem;

  { And a complete tune }
  TTune = record
            Length    : byte;
            FirstElem : PTuneElem;
          end; { record }

  { A tune file on disk }
  TTuneFile = file of TTuneElem;

  { Sound interface definition }
  TSound = object
             public
               { Initializes the speaker and reads all tunes from disk }
               constructor Init;

               { Plays a single random tune }
               procedure PlayTune;

             private
               { All the tunes present }
               AllTunes : array [1..MaxTunes] of TTune;

               { Which tune was previously played }
               JustHad : byte;

               { Gets a single tune from disk }
               procedure ReadTune (var Tune : TTune; var TuneFile : TTuneFile);

           end; { object }

var
  { The internal speaker? }
  Speaker : TSound;

implementation

{ Increases the given tune pointer }
procedure IncPtr (var TuneElem : PTuneElem);

var
  SegPtr,
  OfsPtr : word;

begin
  SegPtr := Seg (TuneElem^);
  OfsPtr := Ofs (TuneElem^);
  Inc (OfsPtr, SizeOf (TTuneElem));
  if OfsPtr < SizeOf (TTuneElem) then begin
    Inc (SegPtr);
  end; { then }
  TuneElem := Ptr (SegPtr, OfsPtr);
end; { IncPtr }


{ Plays a single random tune }
procedure TSound.PlayTune;

var
  TuneNr,
  TuneElemNr,
  TuneLength : byte;
  InterTune  : PTuneElem;

begin
  { Look if it even nescesary to produce sound }
  if not SoundOn then begin
    Exit;
  end; { then }

  repeat
    { Get which tune to play }
    TuneNr := Random (MaxTunes) + 1;
  until TuneNr <> JustHad;

  { Start at the beginning of the tune }
  TuneLength := AllTunes [TuneNr].Length;
  InterTune  := AllTunes [TuneNr].FirstElem;

  { And play the tune until the end }
  for TuneElemNr := 0 to TuneLength do begin
    { Set the speaker to the right amount of Hz }
    Sound (InterTune^.Freq);

    { Wait for the specified amount of time }
    Delay (InterTune^.Delay);

    { And go to the next tune }
    IncPtr (InterTune);
  end; { for }

  { And kill the last note again }
  NoSound;
end; { PlayTune }


{ Gets a single tune from disk }
procedure TSound.ReadTune (var Tune : TTune; var TuneFile : TTuneFile);

var
  TuneElemNr : byte;
  ReadAway   : TTuneElem;
  TuneElem   : PTuneElem;
  Length     : byte;

begin
  { Get the length of the tune }
  Reset (TuneFile);
  Length := 0;
  while not Eof (TuneFile) do begin
    Read (TuneFile, ReadAway);
    Inc (Length);
  end; { while }
  Dec (Length);

  { Make the new tune instance }
  Tune.Length := Length;
  GetMem (Tune.FirstElem, (Length + 1) * SizeOf (TTuneElem));

  { And fill it up with file data }
  Reset (TuneFile);
  TuneElem := Tune.FirstElem;
  for TuneElemNr := 0 to Length do begin
    Read (TuneFile, TuneElem^);
    IncPtr (TuneElem);
  end; { for }
  Close (TuneFile);
end; { ReadTune }

{ Initializes the speaker and reads all tunes from disk }
constructor TSound.Init;

var
  TuneFile   : TTuneFile;
  TuneNr     : byte;
  NumberText : string;

begin
  { Read all the tunes }
  for TuneNr := 1 to MaxTunes do begin
    Str (TuneNr, NumberText);
    Assign (TuneFile, 'ChiFu.T' + NumberText);
    ReadTune (AllTunes [TuneNr], TuneFile);
  end; { for }
  JustHad := 0;
end; { Init }

begin
  Speaker.Init;
end. { CF_Snd }