{-----------------------------------------------------------------------------

 CF_Time : clock definition and implementation of the game ChiFu.

 Consists of:

 Types:
 -TTime  : quick time reference
 -TClock : clock interface definition
   Variables:
   -TickTime : amount of 100th-seconds per clock tick
   -WaitTime : the amount of time to wait (in 100th-seconds)

   Routines:
   -Reset       : resets the clock
   -ClockTicked : returns wether the clock has 'ticked' since the last call
   -Wait        : waits for a brief moment

 Constants:

 Variables:
 -Clock : the clock interface

 Routines:

 Defaults:
 -Resets the clock

-----------------------------------------------------------------------------}
unit CF_Time;

interface

uses
  Dos;

type
  { Quick time reference }
  TTime = record
            Min,
            Sec,
            Sec100 : word;
          end; { record }

  { The internal clock and interface definition }
  TClock = object
            public
              { Amount of 100th-seconds per clock tick }
              TickTime : word;

              { The amount of time to wait (in 100th-seconds) }
              WaitTime : word;

              { Resets the clock }
              procedure Reset;

              { Returns wether the clock has 'ticked' since the last call }
              function ClockTicked : boolean;

              { Waits for a brief moment }
              procedure Wait;

            private

              CurrentTime,
              LastTime    : TTime;

          end; { record }

var
  Clock : TClock;

implementation

{ Resets the clock }
procedure TClock.Reset;

var
  Dummy : word;

begin
  GetTime (Dummy, LastTime.Min, LastTime.Sec, LastTime.Sec100);
end; { Reset }

{ Returns wether the clock has 'ticked' since the last call }
function TClock.ClockTicked : boolean;

var
  TimeDiff : longint;
  Dummy    : word;

begin
  { Get the current time }
  GetTime (Dummy, CurrentTime.Min, CurrentTime.Sec, CurrentTime.Sec100);

  { Calculate the time difference between now and the last clock tick }
  TimeDiff := (CurrentTime.Min - LastTime.Min) * 6000 +
              (CurrentTime.Sec - LastTime.Sec) * 100 +
              (CurrentTime.Sec100 - LastTime.Sec100);

  { And look if enough time has passed for a clock tick }
  if (TimeDiff < 0) or (TimeDiff > TickTime) then begin
    { Clock has ticked -> reset time for next tick }
    Reset;
    ClockTicked := true;
  end { then }
  else begin
    { Clock has not yet ticked }
    ClockTicked := false;
  end; { else }
end; { ClockTicked }

{ Waits for a brief moment }
procedure TClock.Wait;

var
  OldTickTime : word;

begin
  { Remember old tick time }
  OldTickTime := TickTime;

  { Set tick time to waiting time }
  TickTime := WaitTime;

  { Wait for that amount of time }
  Reset;
  repeat
  until ClockTicked;

  { And set the original tick time again }
  TickTime := OldTickTime;
end; { wait }

begin
  Clock.Reset;
end. { CF_Time }